@@ -1,16 +1,22 @@
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
+
 import bisect
 import copy
-import pyatspi
 import time
 from gi.repository import GLib
 
 from . import cmdnames
 from . import chnames
+from . import debug
 from . import keybindings
 from . import messages
 from . import input_event
 from . import orca_state
 from . import settings_manager
+from .ax_collection import AXCollection
+from .ax_object import AXObject
 
 _settingsManager = settings_manager.getManager()
 
@@ -24,14 +30,14 @@ LIVE_RUDE      = 3
 # Seconds a message is held in the queue before it is discarded
 MSG_KEEPALIVE_TIME = 45  # in seconds
 
-# The number of messages that are cached and can later be reviewed via 
+# The number of messages that are cached and can later be reviewed via
 # LiveRegionManager.reviewLiveAnnouncement.
 CACHE_SIZE = 9  # corresponds to one of nine key bindings
 
 class PriorityQueue:
     """ This class represents a thread **UNSAFE** priority queue where priority
-    is determined by the given integer priority.  The entries are also   
-    maintained in chronological order. 
+    is determined by the given integer priority.  The entries are also
+    maintained in chronological order.
 
     TODO: experiment with Queue.Queue to make thread safe
     """
@@ -42,7 +48,7 @@ class PriorityQueue:
         """ Add a new element to the queue according to 1) priority and
         2) timestamp. """
         bisect.insort_left(self.queue, (priority, time.time(), data, obj))
-       
+
     def dequeue(self):
         """get the highest priority element from the queue.  """
         return self.queue.pop(0)
@@ -52,16 +58,21 @@ class PriorityQueue:
         self.queue = []
 
     def purgeByKeepAlive(self):
-        """ Purge items from the queue that are older than the keepalive 
-        time """
+        """ Purge items from the queue that are older than the keepalive time """
         currenttime = time.time()
-        myfilter = lambda item: item[1] + MSG_KEEPALIVE_TIME > currenttime
+
+        def myfilter(item):
+            return item and item[1] + MSG_KEEPALIVE_TIME > currenttime
+
         self.queue = list(filter(myfilter, self.queue))
 
     def purgeByPriority(self, priority):
         """ Purge items from the queue that have a lower than or equal priority
         than the given argument """
-        myfilter = lambda item: item[0] > priority
+
+        def myfilter(item):
+            return item and item[0] > priority
+
         self.queue = list(filter(myfilter, self.queue))
 
     def __len__(self):
@@ -225,6 +236,7 @@ class LiveRegionManager:
         """
 
         if len(self.msg_queue) > 0:
+            debug.printMessage(debug.eventDebugLevel, "\nvvvvv PRESENT LIVE REGION MESSAGE vvvvv")
             self.msg_queue.purgeByKeepAlive()
             politeness, timestamp, message, obj = self.msg_queue.dequeue()
             # Form output message.  No need to repeat labels and content.
@@ -234,7 +246,12 @@ class LiveRegionManager:
                 utts = message['content']
             else:
                 utts = message['labels'] + message['content']
-            self._script.presentMessage(utts)
+
+            if self.monitoring:
+                self._script.presentMessage(utts)
+            else:
+                msg = "INFO: Not presenting message because monitoring is off"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
 
             # set the last live obj to be announced
             self.lastliveobj = obj
@@ -246,6 +263,10 @@ class LiveRegionManager:
         if not self.monitoring:
             self.msg_queue.purgeByKeepAlive()
 
+        msg = f'LIVE REGIONS: messages in queue: {len(self.msg_queue)}'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.eventDebugLevel, "^^^^^ PRESENT LIVE REGION MESSAGE ^^^^^\n")
+
         # See you again soon, stay in event loop if we still have messages.
         return len(self.msg_queue) > 0
 
@@ -332,7 +353,7 @@ class LiveRegionManager:
         if self.monitoring:
             self._script.presentMessage(messages.LIVE_REGIONS_ALL_OFF)
             self.msg_queue.clear()
-            
+
             # First we'll save off a copy for quick restoration
             self._restoreOverrides = copy.copy(self._politenessOverrides)
 
@@ -343,7 +364,7 @@ class LiveRegionManager:
             # look through all the objects on the page and set/add to
             # politeness overrides.  This only adds live regions with good
             # markup.
-            matches = pyatspi.findAllDescendants(docframe, self.matchLiveRegion)
+            matches = self.getAllLiveRegions(docframe)
             for match in matches:
                 objectid = self._getObjectId(match)
                 self._politenessOverrides[(uri, objectid)] = LIVE_OFF
@@ -359,6 +380,19 @@ class LiveRegionManager:
             # Toggle our flag
             self.monitoring = True  
 
+    def getAllLiveRegions(self, document):
+        attrs = []
+        levels = ["off", "polite", "assertive"]
+        for level in levels:
+            attrs.append('container-live:' + level)
+
+        rule = AXCollection.create_match_rule(attributes=attrs)
+        result = AXCollection.get_all_matches(document, rule)
+
+        msg = f'LIVE REGIONS: {len(result)} regions found'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        return result
+
     def generateLiveRegionDescription(self, obj, **args):
         """Used in conjunction with whereAmI to output description and 
         politeness of the given live region object"""
@@ -368,21 +402,20 @@ class LiveRegionManager:
         results = []
 
         # get the description if there is one.
-        for relation in obj.getRelationSet():
-            relationtype = relation.getRelationType()
-            if relationtype == pyatspi.RELATION_DESCRIBED_BY:
-                targetobj = relation.getTarget(0)
-                try:
-                    # We will add on descriptions if they don't duplicate
-                    # what's already in the object's description.
-                    # See http://bugzilla.gnome.org/show_bug.cgi?id=568467
-                    # for more information.
-                    #
-                    description = targetobj.queryText().getText(0, -1)
-                    if description.strip() != obj.description.strip():
-                        results.append(description)
-                except NotImplemented:
-                    pass
+        relation = AXObject.get_relation(obj, Atspi.RelationType.DESCRIBED_BY)
+        if relation:
+            targetobj = relation.getTarget(0)
+            try:
+                # We will add on descriptions if they don't duplicate
+                # what's already in the object's description.
+                # See http://bugzilla.gnome.org/show_bug.cgi?id=568467
+                # for more information.
+                #
+                description = targetobj.queryText().getText(0, -1)
+                if description.strip() != AXObject.get_description(obj).strip():
+                    results.append(description)
+            except NotImplementedError:
+                pass
 
         # get the politeness level as a string
         try:
@@ -398,17 +431,21 @@ class LiveRegionManager:
 
         return results
 
-    def matchLiveRegion(self, obj):
-        """Predicate used to find a live region"""
-        attrs = self._getAttrDictionary(obj)
-        return 'container-live' in attrs
+    def _findContainer(self, obj):
+        def isContainer(x):
+            return self._getAttrDictionary(x).get('atomic')
+
+        if isContainer(obj):
+            return obj
+
+        return AXObject.find_ancestor(obj, isContainer)
 
     def _getMessage(self, event):
         """Gets the message associated with a given live event."""
         attrs = self._getAttrDictionary(event.source)
         content = ""
         labels = ""
-        
+
         # A message is divided into two parts: labels and content.  We
         # will first try to get the content.  If there is None, 
         # assume it is an invalid message and return None
@@ -420,11 +457,14 @@ class LiveRegionManager:
 
         elif event.type.startswith('object:text-changed:insert'):
             if attrs.get('container-atomic') != 'true':
-                content = event.any_data
+                if "\ufffc" not in event.any_data:
+                    content = event.any_data
+                else:
+                    content = self._script.utilities.expandEOCs(
+                        event.source, event.detail1, event.detail1 + event.detail2)
             else:
-                text = self._script.utilities.queryNonEmptyText(event.source)
-                if text:
-                    content = text.getText(0, -1)
+                container = self._findContainer(event.source)
+                content = self._script.utilities.expandEOCs(container)
 
         if not content:
             return None
@@ -436,7 +476,7 @@ class LiveRegionManager:
         # Proper live regions typically come with proper aria labels. These
         # labels are typically exposed as names. Failing that, descriptions.
         # Looking for actual labels seems a non-performant waste of time.
-        name = (event.source.name or event.source.description).strip()
+        name = (AXObject.get_name(event.source) or AXObject.get_description(event.source)).strip()
         if name and name != content:
             labels = name
 
@@ -498,7 +538,8 @@ class LiveRegionManager:
                 return LIVE_ASSERTIVE
             elif attrs['container-live'] == 'rude': 
                 return LIVE_RUDE
-            else: return LIVE_NONE
+            else:
+                return LIVE_NONE
         except KeyError:
             return LIVE_NONE
 
@@ -514,28 +555,23 @@ class LiveRegionManager:
             return 'rude'
         elif politeness == LIVE_NONE: 
             return 'none'
-        else: return 'unknown'
+        else:
+            return 'unknown'
 
     def _getAttrDictionary(self, obj):
-        try:
-            return dict([attr.split(':', 1) for attr in obj.getAttributes()])
-        except:
-            return {}
-    
+        return self._script.utilities.objectAttributes(obj)
+
     def _getPath(self, obj):
         """ Returns, as a tuple of integers, the path from the given object 
         to the document frame."""
         docframe = self._script.utilities.documentFrame()
         path = []
         while True:
-            if obj.parent is None or obj == docframe:
+            if obj == docframe or AXObject.get_parent(obj) is None:
                 path.reverse()
                 return tuple(path)
-            try:
-                path.append(obj.getIndexInParent())
-            except Exception:
-                raise LookupError
-            obj = obj.parent
+            path.append(AXObject.get_index_in_parent(obj))
+            obj = AXObject.get_parent(obj)
 
     def toggleMonitoring(self, script, inputEvent):
         if not _settingsManager.getSetting('inferLiveRegions'):
