@@ -25,15 +25,12 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2013-2014 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
-import time
-
 import orca.debug as debug
-import orca.mouse_review as mouse_review
 import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.scripts.default as default
-import orca.speech as speech
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
 
 from .script_utilities import Utilities
 
@@ -55,15 +52,13 @@ class Script(default.Script):
         """Handles changes of focus of interest to the script."""
 
         if self.utilities.isToggleDescendantOfComboBox(newFocus):
-            isComboBox = lambda x: x and x.getRole() == pyatspi.ROLE_COMBO_BOX
-            newFocus = pyatspi.findAncestor(newFocus, isComboBox) or newFocus
+            newFocus = AXObject.find_ancestor(newFocus, AXUtilities.is_combo_box) or newFocus
             orca.setLocusOfFocus(event, newFocus, False)
         elif self.utilities.isInOpenMenuBarMenu(newFocus):
             window = self.utilities.topLevelObject(newFocus)
             windowChanged = window and orca_state.activeWindow != window
             if windowChanged:
-                orca_state.activeWindow = window
-                self.windowActivateTime = time.time()
+                orca.setActiveWindow(window)
 
         super().locusOfFocusChanged(event, oldFocus, newFocus)
 
@@ -71,12 +66,14 @@ class Script(default.Script):
         """Callback for object:active-descendant-changed accessibility events."""
 
         if not self.utilities.isTypeahead(orca_state.locusOfFocus):
+            msg = "GTK: locusOfFocus is not typeahead. Passing along to default script."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             super().onActiveDescendantChanged(event)
             return
 
         msg = "GTK: locusOfFocus believed to be typeahead. Presenting change."
-        debug.println(debug.LEVEL_INFO, msg, True)
-        self.presentObject(event.any_data)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        self.presentObject(event.any_data, interrupt=True)
 
     def onCheckedChanged(self, event):
         """Callback for object:state-changed:checked accessibility events."""
@@ -87,12 +84,10 @@ class Script(default.Script):
             return
 
         # Present changes of child widgets of GtkListBox items
-        isListBox = lambda x: x and x.getRole() == pyatspi.ROLE_LIST_BOX
-        if not pyatspi.findAncestor(obj, isListBox):
+        if not AXObject.find_ancestor(obj, AXUtilities.is_list_box):
             return
 
-        self.updateBraille(obj)
-        speech.speak(self.speechGenerator.generateSpeech(obj, alreadyFocused=True))
+        self.presentObject(obj, alreadyFocused=True, interrupt=True)
 
     def onFocus(self, event):
         """Callback for focus: accessibility events."""
@@ -107,33 +102,31 @@ class Script(default.Script):
         if self.utilities.isLayoutOnly(event.source):
             return
 
-        if event.source == mouse_review.reviewer.getCurrentItem():
+        if event.source == self.mouseReviewer.getCurrentItem():
             msg = "GTK: Event source is current mouse review item"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         if self.utilities.isTypeahead(orca_state.locusOfFocus) \
-           and "Table" in pyatspi.listInterfaces(event.source) \
-           and not event.source.getState().contains(pyatspi.STATE_FOCUSED):
+           and AXObject.supports_table(event.source) \
+           and not AXUtilities.is_focused(event.source):
             return
 
-        if "Table" in pyatspi.listInterfaces(event.source):
-            selectedChildren = self.utilities.selectedChildren(event.source)
-            if selectedChildren:
-                orca.setLocusOfFocus(event, selectedChildren[0])
-                return
-
-        ancestor = pyatspi.findAncestor(orca_state.locusOfFocus, lambda x: x == event.source)
+        ancestor = AXObject.find_ancestor(orca_state.locusOfFocus, lambda x: x == event.source)
         if not ancestor:
             orca.setLocusOfFocus(event, event.source)
             return
 
-        if ancestor and "Table" in pyatspi.listInterfaces(ancestor):
+        if AXObject.supports_table(ancestor):
             return
 
-        isMenu = lambda x: x and x.getRole() == pyatspi.ROLE_MENU
-        if isMenu(ancestor) and not pyatspi.findAncestor(ancestor, isMenu):
-            return
+        if AXUtilities.is_menu(ancestor):
+            if AXUtilities.is_selected(orca_state.locusOfFocus):
+                msg = "GTK: Event source is ancestor of selected focus. Ignoring."
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
+                return
+            msg = "GTK: Event source is ancestor of unselected focus. Updating focus."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         orca.setLocusOfFocus(event, event.source)
 
@@ -142,7 +135,7 @@ class Script(default.Script):
 
         if self.utilities.isUselessPanel(event.source):
             msg = "GTK: Event source believed to be useless panel"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         super().onFocusedChanged(event)
@@ -158,9 +151,8 @@ class Script(default.Script):
                 orca.setLocusOfFocus(event, None)
                 return
 
-        role = event.source.getRole()
-        if role in [pyatspi.ROLE_CANVAS, pyatspi.ROLE_ICON] \
-           and self.utilities.handleContainerSelectionChange(event.source.parent):
+        if AXUtilities.is_icon_or_canvas(event.source) \
+           and self.utilities.handleContainerSelectionChange(AXObject.get_parent(event.source)):
             return
 
         super().onSelectedChanged(event)
@@ -173,14 +165,13 @@ class Script(default.Script):
             super().onSelectionChanged(event)
             return
 
-        isFocused = event.source.getState().contains(pyatspi.STATE_FOCUSED)
-        role = event.source.getRole()
-        if role == pyatspi.ROLE_COMBO_BOX and not isFocused:
+        isFocused = AXUtilities.is_focused(event.source)
+        if AXUtilities.is_combo_box(event.source) and not isFocused:
             return
 
         if not isFocused and self.utilities.isTypeahead(orca_state.locusOfFocus):
             msg = "GTK: locusOfFocus believed to be typeahead. Presenting change."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
             selectedChildren = self.utilities.selectedChildren(event.source)
             for child in selectedChildren:
@@ -188,7 +179,7 @@ class Script(default.Script):
                     self.presentObject(child)
             return
 
-        if role == pyatspi.ROLE_LAYERED_PANE \
+        if AXUtilities.is_layered_pane(event.source) \
            and self.utilities.selectedChildCount(event.source) > 1:
             return
 
@@ -201,12 +192,12 @@ class Script(default.Script):
             super().onShowingChanged(event)
             return
 
-        obj = event.source
-        if self.utilities.isPopOver(obj) \
-           or obj.getRole() in [pyatspi.ROLE_ALERT, pyatspi.ROLE_INFO_BAR]:
-            if obj.parent and obj.parent.getRole() == pyatspi.ROLE_APPLICATION:
+        if self.utilities.isPopOver(event.source) \
+           or AXUtilities.is_alert(event.source) \
+           or AXUtilities.is_info_bar(event.source):
+            if AXUtilities.is_application(AXObject.get_parent(event.source)):
                 return
-            self.presentObject(event.source)
+            self.presentObject(event.source, interrupt=True)
             return
 
         super().onShowingChanged(event)
@@ -215,8 +206,8 @@ class Script(default.Script):
         """Callback for object:text-changed:delete accessibility events."""
 
         if not self.utilities.isShowingAndVisible(event.source):
-            msg = "GTK: %s is not showing and visible" % event.source
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["GTK:", event.source, "is not showing and visible"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return
 
         super().onTextDeleted(event)
@@ -225,8 +216,8 @@ class Script(default.Script):
         """Callback for object:text-changed:insert accessibility events."""
 
         if not self.utilities.isShowingAndVisible(event.source):
-            msg = "GTK: %s is not showing and visible" % event.source
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["GTK:", event.source, "is not showing and visible"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return
 
         super().onTextInserted(event)
