@@ -27,7 +27,9 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc." \
                 "Copyright (c) 2011-2015 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 import urllib
 
 from orca import debug
@@ -37,6 +39,8 @@ from orca import orca_state
 from orca import settings
 from orca import settings_manager
 from orca import speech_generator
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
 
 _settingsManager = settings_manager.getManager()
 
@@ -81,28 +85,32 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             if priorDoc != doc and not self._script.utilities.getDocumentForObject(doc):
                 result = [super()._generateName(doc)]
 
-        if self._script.utilities.isLink(obj) \
-           or self._script.utilities.isLandmark(obj) \
-           or self._script.utilities.isMath(obj) \
-           or obj.getRole() in [pyatspi.ROLE_TOOL_TIP, pyatspi.ROLE_STATUS_BAR]:
+        if not self._script.utilities.getTable(obj) \
+           and (self._script.utilities.isLink(obj) \
+                or self._script.utilities.isLandmark(obj) \
+                or self._script.utilities.isMath(obj) \
+                or AXUtilities.is_tool_tip(obj) \
+                or AXUtilities.is_status_bar(obj)):
             return result
 
-        args['stopAtRoles'] = [pyatspi.ROLE_DOCUMENT_FRAME,
-                               pyatspi.ROLE_DOCUMENT_WEB,
-                               pyatspi.ROLE_EMBEDDED,
-                               pyatspi.ROLE_INTERNAL_FRAME,
-                               pyatspi.ROLE_MATH,
-                               pyatspi.ROLE_MENU_BAR]
-        args['skipRoles'] = [pyatspi.ROLE_PARAGRAPH,
-                             pyatspi.ROLE_HEADING,
-                             pyatspi.ROLE_LABEL,
-                             pyatspi.ROLE_LINK,
-                             pyatspi.ROLE_LIST_ITEM,
-                             pyatspi.ROLE_TEXT]
-        args['stopAfterRoles'] = [pyatspi.ROLE_TOOL_BAR]
+        if self._script.utilities.isItemForEditableComboBox(obj, priorObj):
+            return result
+
+        args['stopAtRoles'] = [Atspi.Role.DOCUMENT_WEB,
+                               Atspi.Role.EMBEDDED,
+                               Atspi.Role.INTERNAL_FRAME,
+                               Atspi.Role.MATH,
+                               Atspi.Role.MENU_BAR]
+        args['skipRoles'] = [Atspi.Role.PARAGRAPH,
+                             Atspi.Role.HEADING,
+                             Atspi.Role.LABEL,
+                             Atspi.Role.LINK,
+                             Atspi.Role.LIST_ITEM,
+                             Atspi.Role.TEXT]
+        args['stopAfterRoles'] = [Atspi.Role.TOOL_BAR]
 
         if self._script.utilities.isEditableDescendantOfComboBox(obj):
-            args['skipRoles'].append(pyatspi.ROLE_COMBO_BOX)
+            args['skipRoles'].append(Atspi.Role.COMBO_BOX)
 
         result.extend(super()._generateAncestors(obj, **args))
 
@@ -147,7 +155,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             result = [messages.HAS_POPUP_TREE]
 
         if result:
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         return result
 
@@ -158,13 +166,16 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return []
 
+        if self._script.utilities.isFeedArticle(obj):
+            return []
+
         if not args.get('mode', None):
             args['mode'] = self._mode
 
         args['stringType'] = 'clickable'
         if self._script.utilities.isClickableElement(obj):
             result = [self._script.formatting.getString(**args)]
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         return []
@@ -182,9 +193,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.preferDescriptionOverName(obj):
             return []
 
-        role = args.get('role', obj.getRole())
+        role = args.get('role', AXObject.get_role(obj))
         if obj != orca_state.locusOfFocus:
-            if role in [pyatspi.ROLE_ALERT, pyatspi.ROLE_DIALOG]:
+            if role in [Atspi.Role.ALERT, Atspi.Role.DIALOG]:
                 return super()._generateDescription(obj, **args)
             if not args.get('inMouseReview'):
                 return []
@@ -193,11 +204,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if formatType == 'basicWhereAmI' and self._script.utilities.isLiveRegion(obj):
             return self._script.liveRegionManager.generateLiveRegionDescription(obj, **args)
 
-        if role == pyatspi.ROLE_TEXT and formatType != 'basicWhereAmI':
+        if role == Atspi.Role.TEXT and formatType != 'basicWhereAmI':
             return []
 
         # TODO - JD: This is private.
-        if role == pyatspi.ROLE_LINK and self._script._lastCommandWasCaretNav:
+        if role == Atspi.Role.LINK and self._script._lastCommandWasCaretNav:
             return []
 
         return super()._generateDescription(obj, **args)
@@ -215,7 +226,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         args['stringType'] = 'haslongdesc'
         if self._script.utilities.hasLongDesc(obj):
             result = [self._script.formatting.getString(**args)]
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         return []
@@ -231,12 +242,14 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not objs:
             return []
 
-        objString = lambda x: str.strip("%s %s" % (x.name, self.getLocalizedRoleName(x)))
+        def objString(x):
+            return str.strip(f"{AXObject.get_name(x)} {self.getLocalizedRoleName(x)}")
+
         toPresent = ", ".join(set(map(objString, objs)))
 
         args['stringType'] = 'hasdetails'
         result = [self._script.formatting.getString(**args) % toPresent]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     def _generateAllDetails(self, obj, **args):
@@ -245,7 +258,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         objs = self._script.utilities.detailsIn(obj)
         if not objs:
-            container = pyatspi.findAncestor(obj, self._script.utilities.hasDetails)
+            container = AXObject.find_ancestor(obj, self._script.utilities.hasDetails)
             objs = self._script.utilities.detailsIn(container)
 
         if not objs:
@@ -253,19 +266,19 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         args['stringType'] = 'hasdetails'
         result = [self._script.formatting.getString(**args) % ""]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         result = []
         for o in objs:
             result.append(self.getLocalizedRoleName(o))
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
             string = self._script.utilities.expandEOCs(o)
             if not string.strip():
                 continue
 
             result.append(string)
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             result.extend(self._generatePause(o))
 
         return result
@@ -301,7 +314,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 words = words[0:5] + ['...']
 
             result.append(self._script.formatting.getString(**objArgs) % " ".join(words))
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             result.extend(self._generatePause(o, **objArgs))
 
         return result
@@ -324,17 +337,31 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if priorObj and priorObj in self._script.utilities.labelsForObject(obj):
             return []
 
+        objName = AXObject.get_name(obj)
+        descendant = args.get("ancestorOf")
+        if descendant and priorObj and objName and objName == AXObject.get_name(priorObj):
+            tokens = ["WEB: ", descendant, "'s ancestor", obj, "has same name as priorObj",
+                      priorObj, ". Not generating labelOrName."]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return []
+
+        role = args.get('role', AXObject.get_role(obj))
+        if role == Atspi.Role.MENU and self._script.utilities.isPopupMenuForCurrentItem(obj):
+            tokens = ["WEB: ", obj, "is popup menu for current item."]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return []
+
         if self._script.utilities.isContentEditableWithEmbeddedObjects(obj) \
            or self._script.utilities.isDocument(obj):
             lastKey, mods = self._script.utilities.lastKeyAndModifiers()
             if lastKey in ["Home", "End", "Up", "Down", "Left", "Right", "Page_Up", "Page_Down"]:
                 return []
 
-        if priorObj and priorObj.getRole() == pyatspi.ROLE_PAGE_TAB and priorObj.name == obj.name:
+        if AXUtilities.is_page_tab(priorObj) and AXObject.get_name(priorObj) == objName:
             return []
 
-        if obj.name:
-            name = obj.name
+        if objName:
+            name = objName
             if not self._script.utilities.hasExplicitName(obj):
                 name = name.strip()
 
@@ -342,11 +369,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 name = self._script.utilities.verbalizeAllPunctuation(name)
 
             result = [name]
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
-        if obj.getRole() == pyatspi.ROLE_CHECK_BOX:
-            gridCell = pyatspi.findAncestor(obj, self._script.utilities.isGridCell)
+        if AXUtilities.is_check_box(obj):
+            gridCell = AXObject.find_ancestor(obj, self._script.utilities.isGridCell)
             if gridCell:
                 return super()._generateLabelOrName(gridCell, **args)
 
@@ -367,32 +394,31 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         if self._script.utilities.isFigure(obj) and args.get('ancestorOf'):
             caption = args.get('ancestorOf')
-            if caption.getRole() != pyatspi.ROLE_CAPTION:
-                isCaption = lambda x: x and x.getRole() == pyatspi.ROLE_CAPTION
-                caption = pyatspi.findAncestor(caption, isCaption)
+            if not AXUtilities.is_caption(caption):
+                caption = AXObject.find_ancestor(caption, AXUtilities.is_caption)
             if caption and hash(obj) in self._script.utilities.labelTargets(caption):
                 return []
 
-        role = args.get('role', obj.getRole())
+        role = args.get('role', AXObject.get_role(obj))
 
         # TODO - JD: Once the formatting strings are vastly cleaned up
         # or simply removed, hacks like this won't be needed.
-        if role in [pyatspi.ROLE_COMBO_BOX, pyatspi.ROLE_SPIN_BUTTON]:
+        if role in [Atspi.Role.COMBO_BOX, Atspi.Role.SPIN_BUTTON]:
             return super()._generateName(obj, **args)
 
-        if obj.name:
+        if AXObject.get_name(obj):
             if self._script.utilities.preferDescriptionOverName(obj):
-                result = [obj.description]
+                result = [AXObject.get_description(obj)]
             elif self._script.utilities.isLink(obj) \
                  and not self._script.utilities.hasExplicitName(obj):
                 return []
             else:
-                name = obj.name
+                name = AXObject.get_name(obj)
                 if not self._script.utilities.hasExplicitName(obj):
                     name = name.strip()
                 result = [name]
 
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
         return super()._generateName(obj, **args)
@@ -407,7 +433,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         label, objects = self._script.utilities.inferLabelFor(obj)
         if label:
             result = [label]
-            result.extend(self.voice(speech_generator.DEFAULT))
+            result.extend(self.voice(speech_generator.DEFAULT, obj=obj, **args))
             return result
 
         return super()._generateLabel(obj, **args)
@@ -432,15 +458,17 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if self._script.utilities.inDocumentContent(obj) \
            and not self._script.utilities.inDocumentContent(orca_state.locusOfFocus):
             result = ['']
-            result.extend(self.voice(speech_generator.SYSTEM))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
         return super()._generateLeaving(obj, **args)
 
     def _generateNewRadioButtonGroup(self, obj, **args):
-        # TODO - JD: Looking at the default speech generator's method, this
-        # is all kinds of broken. Until that can be sorted out, try to filter
-        # out some of the noise....
+        # TODO - JD: The default speech generator's method determines group membership
+        # via the member-of relation. We cannot count on that here. Plus, radio buttons
+        # on the web typically live in a group which is labelled. Thus the new-ancestor
+        # presentation accomplishes the same thing. Unless this can be further sorted out,
+        # try to filter out some of the noise....
         return []
 
     def _generateNumberOfChildren(self, obj, **args):
@@ -451,20 +479,32 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         # We handle things even for non-document content due to issues in
         # other toolkits (e.g. exposing list items to us that are not
         # exposed to sighted users)
-        role = args.get('role', obj.getRole())
-        if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
+        roles = [Atspi.Role.DESCRIPTION_LIST,
+                 Atspi.Role.LIST,
+                 Atspi.Role.LIST_BOX,
+                 'ROLE_FEED']
+        role = args.get('role', AXObject.get_role(obj))
+        if role not in roles:
             return super()._generateNumberOfChildren(obj, **args)
 
-        setsize = self._script.utilities.getSetSize(obj[0])
+        setsize = self._script.utilities.getSetSize(AXObject.get_child(obj, 0))
         if setsize is None:
-            children = [x for x in obj if x.getRole() == pyatspi.ROLE_LIST_ITEM]
+            if self._script.utilities.isDescriptionList(obj):
+                children = self._script.utilities.descriptionListTerms(obj)
+            elif role in [Atspi.Role.LIST, Atspi.Role.LIST_BOX]:
+                children = [x for x in AXObject.iter_children(obj, AXUtilities.is_list_item)]
             setsize = len(children)
 
         if not setsize:
             return []
 
-        result = [messages.listItemCount(setsize)]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        if self._script.utilities.isDescriptionList(obj):
+            result = [messages.descriptionListTermCount(setsize)]
+        elif role == 'ROLE_FEED':
+            result = [messages.feedArticleCount(setsize)]
+        else:
+            result = [messages.listItemCount(setsize)]
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     # TODO - JD: Yet another dumb generator method we should kill.
@@ -485,7 +525,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return super()._generateRealActiveDescendantDisplayedText(obj, **args)
 
-        return self._generateDisplayedText(obj, **args)
+        rad = self._script.utilities.realActiveDescendant(obj)
+        return self._generateDisplayedText(rad, **args)
 
     def _generateRoleName(self, obj, **args):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
@@ -498,15 +539,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return []
 
         result = []
-        acss = self.voice(speech_generator.SYSTEM)
-
         roledescription = self._script.utilities.getRoleDescription(obj)
         if roledescription:
             result = [roledescription]
-            result.extend(acss)
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             return result
 
-        role = args.get('role', obj.getRole())
+        role = args.get('role', AXObject.get_role(obj))
         enabled, disabled = self._getEnabledAndDisabledContextRoles()
         if role in disabled:
             return []
@@ -518,39 +557,39 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         total = args.get('total', 1)
 
         if not force:
-            doNotSpeak = [pyatspi.ROLE_FOOTER,
-                          pyatspi.ROLE_FORM,
-                          pyatspi.ROLE_LABEL,
-                          pyatspi.ROLE_MENU_ITEM,
-                          pyatspi.ROLE_PARAGRAPH,
-                          pyatspi.ROLE_SECTION,
-                          pyatspi.ROLE_REDUNDANT_OBJECT,
-                          pyatspi.ROLE_UNKNOWN]
+            doNotSpeak = [Atspi.Role.FOOTER,
+                          Atspi.Role.FORM,
+                          Atspi.Role.LABEL,
+                          Atspi.Role.MENU_ITEM,
+                          Atspi.Role.PARAGRAPH,
+                          Atspi.Role.SECTION,
+                          Atspi.Role.REDUNDANT_OBJECT,
+                          Atspi.Role.UNKNOWN]
         else:
-            doNotSpeak = [pyatspi.ROLE_UNKNOWN]
+            doNotSpeak = [Atspi.Role.UNKNOWN]
 
         if not force:
-            doNotSpeak.append(pyatspi.ROLE_TABLE_CELL)
-            doNotSpeak.append(pyatspi.ROLE_TEXT)
-            doNotSpeak.append(pyatspi.ROLE_STATIC)
+            doNotSpeak.append(Atspi.Role.TABLE_CELL)
+            doNotSpeak.append(Atspi.Role.TEXT)
+            doNotSpeak.append(Atspi.Role.STATIC)
             if args.get('string'):
                 doNotSpeak.append("ROLE_CONTENT_SUGGESTION")
             if args.get('formatType', 'unfocused') != 'basicWhereAmI':
-                doNotSpeak.append(pyatspi.ROLE_LIST_ITEM)
-                doNotSpeak.append(pyatspi.ROLE_LIST)
+                doNotSpeak.append(Atspi.Role.LIST_ITEM)
+                doNotSpeak.append(Atspi.Role.LIST)
             if (start or end):
-                doNotSpeak.append(pyatspi.ROLE_DOCUMENT_FRAME)
-                doNotSpeak.append(pyatspi.ROLE_DOCUMENT_WEB)
-                doNotSpeak.append(pyatspi.ROLE_ALERT)
+                doNotSpeak.append(Atspi.Role.DOCUMENT_FRAME)
+                doNotSpeak.append(Atspi.Role.DOCUMENT_WEB)
+                doNotSpeak.append(Atspi.Role.ALERT)
             if self._script.utilities.isAnchor(obj):
-                doNotSpeak.append(obj.getRole())
+                doNotSpeak.append(AXObject.get_role(obj))
             if total > 1:
-                doNotSpeak.append(pyatspi.ROLE_ROW_HEADER)
+                doNotSpeak.append(Atspi.Role.ROW_HEADER)
             if self._script.utilities.isMenuInCollapsedSelectElement(obj):
-                doNotSpeak.append(pyatspi.ROLE_MENU)
+                doNotSpeak.append(Atspi.Role.MENU)
 
         lastKey, mods = self._script.utilities.lastKeyAndModifiers()
-        isEditable = obj.getState().contains(pyatspi.STATE_EDITABLE)
+        isEditable = AXUtilities.is_editable(obj)
 
         if isEditable and not self._script.utilities.isContentEditableWithEmbeddedObjects(obj):
             if ((lastKey in ["Down", "Right"] and not mods) or self._script.inSayAll()) and start:
@@ -561,49 +600,53 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     return []
             if role not in doNotSpeak:
                 result.append(self.getLocalizedRoleName(obj, **args))
-                result.extend(acss)
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif isEditable and self._script.utilities.isDocument(obj):
-            if obj.parent and not obj.parent.getState().contains(pyatspi.STATE_EDITABLE) \
-               and lastKey not in ["Home", "End", "Up", "Down", "Left", "Right", "Page_Up", "Page_Down"]:
+            parent = AXObject.get_parent(obj)
+            if parent and not AXUtilities.is_editable(parent) \
+               and lastKey not in \
+                    ["Home", "End", "Up", "Down", "Left", "Right", "Page_Up", "Page_Down"]:
                 result.append(object_properties.ROLE_EDITABLE_CONTENT)
-                result.extend(acss)
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
-        elif role == pyatspi.ROLE_HEADING:
+        elif role == Atspi.Role.HEADING:
             if index == total - 1 or not self._script.utilities.isFocusableWithMathChild(obj):
                 level = self._script.utilities.headingLevel(obj)
                 if level:
                     result.append(object_properties.ROLE_HEADING_LEVEL_SPEECH % {
                         'role': self.getLocalizedRoleName(obj, **args),
                         'level': level})
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
                 else:
                     result.append(self.getLocalizedRoleName(obj, **args))
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif self._script.utilities.isLink(obj):
-            if obj.parent.getRole() == pyatspi.ROLE_IMAGE:
+            if AXUtilities.is_image(AXObject.get_parent(obj)):
                 result.append(messages.IMAGE_MAP_LINK)
-                result.extend(acss)
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             else:
                 if self._script.utilities.hasUselessCanvasDescendant(obj):
-                    result.append(self.getLocalizedRoleName(obj, role=pyatspi.ROLE_IMAGE))
-                    result.extend(acss)
+                    result.append(self.getLocalizedRoleName(obj, role=Atspi.Role.IMAGE))
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
                 if index == total - 1 or not self._script.utilities.isFocusableWithMathChild(obj):
                     result.append(self.getLocalizedRoleName(obj, **args))
-                    result.extend(acss)
+                    result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         elif role not in doNotSpeak and args.get('priorObj') != obj:
             result.append(self.getLocalizedRoleName(obj, **args))
-            result.extend(acss)
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         if self._script.utilities.isMath(obj) and not self._script.utilities.isMathTopLevel(obj):
             return result
 
-        ancestorRoles = [pyatspi.ROLE_HEADING, pyatspi.ROLE_LINK]
-        speakRoles = lambda x: x and x.getRole() in ancestorRoles
-        ancestor = pyatspi.findAncestor(obj, speakRoles)
-        if ancestor and ancestor.getRole() != role and (index == total - 1 or obj.name == ancestor.name):
+        def speakRoles(x):
+            return AXUtilities.is_heading(x) or AXUtilities.is_link(x)
+
+        ancestor = AXObject.find_ancestor(obj, speakRoles)
+        if ancestor and AXObject.get_role(ancestor) != role \
+            and (index == total - 1 or AXObject.get_name(obj) == AXObject.get_name(ancestor)):
             result.extend(self._generateRoleName(ancestor))
 
         return result
@@ -619,7 +662,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             return []
 
         result = [string]
-        result.extend(self.voice(speech_generator.SYSTEM))
+        result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
         return result
 
     def _generateSiteDescription(self, obj, **args):
@@ -653,7 +696,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.append(messages.LINK_DIFFERENT_SITE)
 
         if result:
-            result.extend(self.voice(speech_generator.HYPERLINK))
+            result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         return result
 
@@ -670,49 +713,17 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         return result
 
     def _generatePositionInList(self, obj, **args):
-        if _settingsManager.getSetting('onlySpeakDisplayedText'):
-            return []
-
-        if not args.get('forceList', False) \
-           and not _settingsManager.getSetting('enablePositionSpeaking'):
-            return []
-
-        if not self._script.utilities.inDocumentContent(obj):
-            return super()._generatePositionInList(obj, **args)
-
-        menuRoles = [pyatspi.ROLE_MENU_ITEM,
-                     pyatspi.ROLE_TEAROFF_MENU_ITEM,
-                     pyatspi.ROLE_CHECK_MENU_ITEM,
-                     pyatspi.ROLE_RADIO_MENU_ITEM,
-                     pyatspi.ROLE_MENU]
-        if obj.getRole() in menuRoles:
-            return super()._generatePositionInList(obj, **args)
-
-        if self._script.utilities.isEditableComboBox(obj):
-            return []
+        if AXUtilities.is_list_item(obj):
+            thisObjIndex = args.get('index', 0)
+            objCount = args.get('total', 1)
+            if thisObjIndex + 1 < objCount:
+                return []
 
         if args.get('formatType') not in ['basicWhereAmI', 'detailedWhereAmI']:
             if args.get('priorObj') == obj:
                 return []
 
-        position = self._script.utilities.getPositionInSet(obj)
-        total = self._script.utilities.getSetSize(obj)
-        if position is None or total is None:
-            return super()._generatePositionInList(obj, **args)
-
-        position = int(position)
-        total = int(total)
-        if position < 0 or total < 0:
-            return []
-
-        result = []
-        result.append(self._script.formatting.getString(
-            mode='speech',
-            stringType='groupindex') \
-            % {"index" : position,
-               "total" : total})
-        result.extend(self.voice(speech_generator.SYSTEM))
-        return result
+        return super()._generatePositionInList(obj, **args)
 
     def _generateUnselectedCell(self, obj, **args):
         if not self._script.inFocusMode():
@@ -729,16 +740,16 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             label = self._script.utilities.labelForCellCoordinates(obj)
             if label:
                 result.append(label)
-                result.extend(self.voice(speech_generator.SYSTEM))
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
                 return result
 
             row, col = self._script.utilities.coordinatesForCell(obj)
             if self._script.utilities.cellRowChanged(obj):
                 result.append(messages.TABLE_ROW % (row + 1))
-                result.extend(self.voice(speech_generator.SYSTEM))
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
             if self._script.utilities.cellColumnChanged(obj):
                 result.append(messages.TABLE_COLUMN % (col + 1))
-                result.extend(self.voice(speech_generator.SYSTEM))
+                result.extend(self.voice(speech_generator.SYSTEM, obj=obj, **args))
 
         return result
 
@@ -749,9 +760,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if not self._script.utilities.shouldReadFullRow(obj):
             return self._generateRealTableCell(obj, **args)
 
-        isRow = lambda x: x and x.getRole() == pyatspi.ROLE_TABLE_ROW
-        row = pyatspi.findAncestor(obj, isRow)
-        if row and row.name and not self._script.utilities.isLayoutOnly(row):
+        row = AXObject.find_ancestor(obj, AXUtilities.is_table_row)
+        if row and AXObject.get_name(row) and not self._script.utilities.isLayoutOnly(row):
             return self.generate(row)
 
         return super()._generateTableCellRow(obj, **args)
@@ -764,35 +774,40 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
     def generateSpeech(self, obj, **args):
         if not self._script.utilities.inDocumentContent(obj):
-            msg = "WEB: %s is not in document content. Calling default speech generator." % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["WEB:", obj, "is not in document content. Calling default speech generator."]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return super().generateSpeech(obj, **args)
 
-        msg = "WEB: Generating speech for document object %s" % obj
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["WEB: Generating speech for document object", obj]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         result = []
         if args.get('formatType') == 'detailedWhereAmI':
             oldRole = self._overrideRole('default', args)
         elif self._script.utilities.isLink(obj):
-            oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
+            oldRole = self._overrideRole(Atspi.Role.LINK, args)
         elif self._script.utilities.isCustomImage(obj):
-            oldRole = self._overrideRole(pyatspi.ROLE_IMAGE, args)
+            oldRole = self._overrideRole(Atspi.Role.IMAGE, args)
         elif self._script.utilities.treatAsDiv(obj, offset=args.get('startOffset')):
-            oldRole = self._overrideRole(pyatspi.ROLE_SECTION, args)
+            oldRole = self._overrideRole(Atspi.Role.SECTION, args)
         else:
             oldRole = self._overrideRole(self._getAlternativeRole(obj, **args), args)
 
-        if not 'priorObj' in args:
+        if 'priorObj' not in args:
             document = self._script.utilities.getTopLevelDocumentForObject(obj)
             args['priorObj'] = self._script.utilities.getPriorContext(document)[0]
 
+        start = args.get("startOffset", 0)
+        end = args.get("endOffset", -1)
+        args["language"], args["dialect"] = \
+            self._script.utilities.getLanguageAndDialectForSubstring(obj, start, end)
+
         if not result:
             result = list(filter(lambda x: x, super().generateSpeech(obj, **args)))
 
         self._restoreRole(oldRole, args)
-        msg = "WEB: Speech generation for document object %s complete." % obj
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["WEB: Speech generation for document object", obj, "complete."]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return result
 
     def generateContents(self, contents, **args):
@@ -801,18 +816,19 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         result = []
         contents = self._script.utilities.filterContentsForPresentation(contents, True)
-        msg = "WEB: Generating speech contents (length: %i)" % len(contents)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["WEB: Generating speech contents (length:", len(contents), ")"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         for i, content in enumerate(contents):
             obj, start, end, string = content
-            msg = "ITEM %i: %s, start: %i, end: %i, string: '%s'" \
-                  % (i, obj, start, end, string)
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["ITEM", i, ": ", obj, "start: ", start, ", end: ", end, "'", string, "'"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             utterance = self.generateSpeech(
                 obj, startOffset=start, endOffset=end, string=string,
                 index=i, total=len(contents), **args)
             if isinstance(utterance, list):
-                isNotEmptyList = lambda x: not (isinstance(x, list) and not x)
+                def isNotEmptyList(x):
+                    return not (isinstance(x, list) and not x)
+
                 utterance = list(filter(isNotEmptyList, utterance))
             if utterance and utterance[0]:
                 result.append(utterance)
@@ -820,10 +836,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
 
         if not result:
             if self._script.inSayAll(treatInterruptedAsIn=False) \
-               or not _settingsManager.getSetting('speakBlankLines'):
+               or not _settingsManager.getSetting('speakBlankLines') \
+               or args.get('formatType') == 'ancestor':
                 string = ""
             else:
                 string = messages.BLANK
-            result = [string, self.voice(speech_generator.DEFAULT)]
+            result = [string, self.voice(speech_generator.DEFAULT, **args)]
 
         return result
