@@ -28,7 +28,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010 Consorcio Fernando de los Rios."
 __license__   = "LGPL"
 
-import imp
 import importlib
 import os
 from gi.repository import Gio, GLib
@@ -39,6 +38,7 @@ from . import script_manager
 from . import settings
 from . import pronunciation_dict
 from .acss import ACSS
+from .ax_object import AXObject
 from .keybindings import KeyBinding
 
 try:
@@ -50,7 +50,7 @@ try:
         '/org/a11y/bus',
         'org.freedesktop.DBus.Properties',
         None)
-except:
+except Exception:
     _proxy = None
 
 _scriptManager = script_manager.getManager()
@@ -73,7 +73,7 @@ class SettingsManager(object):
         backend='json'
         """
 
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initializing', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initializing', True)
 
         self.backendModule = None
         self._backend = None
@@ -116,10 +116,10 @@ class SettingsManager(object):
         # For handling the currently-"classic" application settings
         self.settingsPackages = ["app-settings"]
 
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initialized', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initialized', True)
 
     def activate(self, prefsDir=None, customSettings={}):
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activating', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activating', True)
 
         self.customizedSettings.update(customSettings)
         self._prefsDir = prefsDir \
@@ -142,16 +142,16 @@ class SettingsManager(object):
         #
         self._createDefaults()
 
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activated', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activated', True)
 
         # Set the active profile and load its stored settings
-        msg = 'SETTINGS MANAGER: Current profile is %s' % self.profile
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SETTINGS MANAGER: Current profile is", self.profile]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         if self.profile is None:
             self.profile = self.general.get('startingProfile')[1]
-            msg = 'SETTINGS MANAGER: Current profile is now %s' % self.profile
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["SETTINGS MANAGER: Current profile is now", self.profile]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         self.setProfile(self.profile)
 
@@ -159,10 +159,10 @@ class SettingsManager(object):
         """Load specific backend for manage user settings"""
 
         try:
-            backend = '.backends.%s_backend' % self.backendName
+            backend = f'.backends.{self.backendName}_backend'
             self.backendModule = importlib.import_module(backend, 'orca')
             return True
-        except:
+        except Exception:
             return False
 
     def _createDefaults(self):
@@ -225,7 +225,7 @@ class SettingsManager(object):
             if value is None:
                 try:
                     value = getattr(settings, key)
-                except:
+                except Exception:
                     pass
             self.defaultGeneral[key] = value
 
@@ -251,22 +251,26 @@ class SettingsManager(object):
 
         success = False
         pathList = [self._prefsDir]
+        tokens = ["SETTINGS MANAGER: Attempt to load orca-customizations"]
+        module_path = pathList[0] + "/orca-customizations.py"
+
         try:
-            msg = "SETTINGS MANAGER: Attempt to load orca-customizations "
-            (fileHnd, moduleName, desc) = \
-                imp.find_module("orca-customizations", pathList)
-            msg += "from %s " % moduleName
-            imp.load_module("orca-customizations", fileHnd, moduleName, desc)
-        except ImportError:
+            spec = importlib.util.spec_from_file_location("orca-customizations", module_path)
+            if spec is not None:
+                module = importlib.util.module_from_spec(spec)
+                spec.loader.exec_module(module)
+                tokens.extend(["from", module_path, "succeeded."])
+                success = True
+            else:
+                tokens.extend(["from", module_path, "failed. Spec not found."])
+        except FileNotFoundError:
+            tokens.extend(["from", module_path, "failed. File not found."])
+        except Exception as error:
+            # Treat this failure as a "success" so that we don't stomp on the existing file.
             success = True
-            msg += "failed due to ImportError. Giving up."
-        except AttributeError:
-            return False
-        else:
-            msg += "succeeded."
-            fileHnd.close()
-            success = True
-        debug.println(debug.LEVEL_ALL, msg, True)
+            tokens.extend(["failed due to:", error, ". Not loading customizations."])
+
+        debug.printTokens(debug.LEVEL_ALL, tokens, True)
         return success
 
     def getPrefsDir(self):
@@ -284,17 +288,33 @@ class SettingsManager(object):
         lang = v.getLocale()
         dialect = v.getDialect()
         if dialect and len(str(dialect)) == 2:
-            lang = "%s_%s" % (lang, dialect.upper())
+            lang = f"{lang}_{dialect.upper()}"
         return lang
 
+    def getSpeechServerFactories(self):
+        """Imports all known SpeechServer factory modules."""
+
+        factories = []
+        for moduleName in self.getSetting('speechFactoryModules'):
+            try:
+                module = importlib.import_module(f'orca.{moduleName}')
+                factories.append(module)
+                tokens = ["SETTINGS MANAGER: Valid speech server factory:", moduleName]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            except Exception:
+                tokens = ["SETTINGS MANAGER: Invalid speech server factory:", moduleName]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
+        return factories
+
     def _loadProfileSettings(self, profile=None):
         """Get from the active backend all the settings for the current
         profile and store them in the object's attributes.
         A profile can be passed as a parameter. This could be useful for
         change from one profile to another."""
 
-        msg = 'SETTINGS MANAGER: Loading settings for %s profile' % profile
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SETTINGS MANAGER: Loading settings for", profile, "profile"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         if profile is None:
             profile = self.profile
@@ -302,15 +322,15 @@ class SettingsManager(object):
         self.profilePronunciations = self.getPronunciations(profile) or {}
         self.profileKeybindings = self.getKeybindings(profile) or {}
 
-        msg = 'SETTINGS MANAGER: Settings for %s profile loaded' % profile
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SETTINGS MANAGER: Settings for", profile, "profile loaded"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     def _mergeSettings(self):
         """Update the changed values on the profile settings
         over the current and active settings"""
 
         msg = 'SETTINGS MANAGER: Merging settings.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         self.profileGeneral.update(self._appGeneral)
         self.profilePronunciations.update(self._appPronunciations)
@@ -321,7 +341,7 @@ class SettingsManager(object):
         self.keybindings.update(self.profileKeybindings)
 
         msg = 'SETTINGS MANAGER: Settings merged.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _enableAccessibility(self):
         """Enables the GNOME accessibility flag.  Users need to log out and
@@ -339,7 +359,7 @@ class SettingsManager(object):
 
     def isAccessibilityEnabled(self):
         msg = 'SETTINGS MANAGER: Checking if accessibility is enabled.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         msg = 'SETTINGS MANAGER: Accessibility enabled: '
         if not _proxy:
@@ -349,23 +369,23 @@ class SettingsManager(object):
             rv = _proxy.Get('(ss)', 'org.a11y.Status', 'IsEnabled')
             msg += str(rv)
 
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return rv
 
     def setAccessibility(self, enable):
-        msg = 'SETTINGS MANAGER: Attempting to set accessibility to %s.' % enable
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f'SETTINGS MANAGER: Attempting to set accessibility to {enable}.'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         if not _proxy:
             msg = 'SETTINGS MANAGER: Error (no proxy)'
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return False
 
         vEnable = GLib.Variant('b', enable)
         _proxy.Set('(ssv)', 'org.a11y.Status', 'IsEnabled', vEnable)
 
-        msg = 'SETTINGS MANAGER: Finished setting accessibility to %s.' % enable
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f'SETTINGS MANAGER: Finished setting accessibility to {enable}.'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def isScreenReaderServiceEnabled(self):
         """Returns True if the screen reader service is enabled. Note that
@@ -381,7 +401,7 @@ class SettingsManager(object):
             rv = _proxy.Get('(ss)', 'org.a11y.Status', 'ScreenReaderEnabled')
             msg += str(rv)
 
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return rv
 
     def setStartingProfile(self, profile=None):
@@ -397,11 +417,10 @@ class SettingsManager(object):
         Also the settings from that profile will be loading
         and updated the current settings with them."""
 
-        msg = 'SETTINGS MANAGER: Setting profile to: %s' % profile
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SETTINGS MANAGER: Setting profile to:", profile]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         oldVoiceLocale = self.getVoiceLocale('default')
-
         self.profile = profile
         self._loadProfileSettings(profile)
         self._mergeSettings()
@@ -416,15 +435,15 @@ class SettingsManager(object):
             orca_i18n.setLocaleForMessages(newVoiceLocale)
             orca_i18n.setLocaleForGUI(newVoiceLocale)
 
-        msg = 'SETTINGS MANAGER: Profile set to: %s' % profile
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SETTINGS MANAGER: Profile set to:", profile]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     def removeProfile(self, profile):
         self._backend.removeProfile(profile)
 
     def _setSettingsRuntime(self, settingsDict):
         msg = 'SETTINGS MANAGER: Setting runtime settings.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         for key, value in settingsDict.items():
             setattr(settings, str(key), value)
@@ -433,7 +452,7 @@ class SettingsManager(object):
             setattr(settings, str(key), value)
 
         msg = 'SETTINGS MANAGER: Runtime settings set.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _setPronunciationsRuntime(self, pronunciationsDict):
         pronunciation_dict.pronunciation_dict = {}
@@ -464,7 +483,7 @@ class SettingsManager(object):
         as the profile's."""
 
         msg = 'SETTINGS MANAGER: Setting general settings for profile'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         self.profileGeneral = {}
 
@@ -479,33 +498,33 @@ class SettingsManager(object):
                 self.profileGeneral[key] = value
 
         msg = 'SETTINGS MANAGER: General settings for profile set'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _setProfilePronunciations(self, pronunciations):
         """Set the changed pronunciations settings from the defaults' ones
         as the profile's."""
 
         msg = 'SETTINGS MANAGER: Setting pronunciation settings for profile.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         self.profilePronunciations = self.defaultPronunciations.copy()
         self.profilePronunciations.update(pronunciations)
 
         msg = 'SETTINGS MANAGER: Pronunciation settings for profile set.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _setProfileKeybindings(self, keybindings):
         """Set the changed keybindings settings from the defaults' ones
         as the profile's."""
 
         msg = 'SETTINGS MANAGER: Setting keybindings settings for profile.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         self.profileKeybindings = self.defaultKeybindings.copy()
         self.profileKeybindings.update(keybindings)
 
         msg = 'SETTINGS MANAGER: Keybindings settings for profile set.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _saveAppSettings(self, appName, general, pronunciations, keybindings):
         appGeneral = {}
@@ -535,12 +554,11 @@ class SettingsManager(object):
     def saveSettings(self, script, general, pronunciations, keybindings):
         """Save the settings provided for the script provided."""
 
-        msg = 'SETTINGS MANAGER: Saving settings for %s (app: %s)' % (script, script.app)
-        debug.println(debug.LEVEL_INFO, msg, True)
-
+        tokens = ["SETTINGS MANAGER: Saving settings for", script, "(app:", script.app, ")"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         app = script.app
         if app:
-            self._saveAppSettings(app.name, general, pronunciations, keybindings)
+            self._saveAppSettings(AXObject.get_name(app), general, pronunciations, keybindings)
             return
 
         # Assign current profile
@@ -557,17 +575,16 @@ class SettingsManager(object):
         self._setProfilePronunciations(pronunciations)
         self._setProfileKeybindings(keybindings)
 
-        msg = 'SETTINGS MANAGER: Saving for backend %s' % self._backend
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SETTINGS MANAGER: Saving for backend", self._backend]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         self._backend.saveProfileSettings(self.profile,
                                           self.profileGeneral,
                                           self.profilePronunciations,
                                           self.profileKeybindings)
 
-        msg = 'SETTINGS MANAGER: Settings for %s (app: %s) saved' % (script, script.app)
-        debug.println(debug.LEVEL_INFO, msg, True)
-
+        tokens = ["SETTINGS MANAGER: Settings for", script, "(app:", script.app, ") saved"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return self._enableAccessibility()
 
     def _adjustBindingTupleValues(self, bindingTuple):
@@ -616,7 +633,7 @@ class SettingsManager(object):
         if not app:
             return None
 
-        appPrefs = self._backend.getAppSettings(app.name)
+        appPrefs = self._backend.getAppSettings(AXObject.get_name(app))
         profiles = appPrefs.get('profiles', {})
         profilePrefs = profiles.get(self.profile, {})
         general = profilePrefs.get('general', {})
@@ -640,14 +657,14 @@ class SettingsManager(object):
         for key in self._appPronunciations.keys():
             self.pronunciations.pop(key)
 
-        prefs = self._backend.getAppSettings(script.app.name)
+        prefs = self._backend.getAppSettings(AXObject.get_name(script.app))
         profiles = prefs.get('profiles', {})
         profilePrefs = profiles.get(self.profile, {})
 
         self._appGeneral = profilePrefs.get('general', {})
         self._appKeybindings = profilePrefs.get('keybindings', {})
         self._appPronunciations = profilePrefs.get('pronunciations', {})
-        self._activeApp = script.app.name
+        self._activeApp = AXObject.get_name(script.app)
 
         self._loadProfileSettings()
         self._mergeSettings()
