@@ -31,9 +31,9 @@ import time
 
 from . import debug
 from . import logger
-from . import orca_state
 from . import settings
 from . import speech_generator
+from .speechserver import VoiceFamily
 
 from .acss import ACSS
 
@@ -44,23 +44,8 @@ log = _logger.newLog("speech")
 #
 _speechserver = None
 
-def getSpeechServerFactories():
-    """Imports all known SpeechServer factory modules.  Returns a list
-    of modules that implement the getSpeechServers method, which
-    returns a list of speechserver.SpeechServer instances.
-    """
-
-    factories = []
-
-    moduleNames = settings.speechFactoryModules
-    for moduleName in moduleNames:
-        try:
-            module = importlib.import_module('orca.%s' % moduleName)
-            factories.append(module)
-        except:
-            debug.printException(debug.LEVEL_CONFIGURATION)
-
-    return factories
+# The last time something was spoken.
+_timestamp = 0
 
 def _initSpeechServer(moduleName, speechServerInfo):
 
@@ -71,11 +56,11 @@ def _initSpeechServer(moduleName, speechServerInfo):
 
     factory = None
     try:
-        factory = importlib.import_module('orca.%s' % moduleName)
-    except:
+        factory = importlib.import_module(f'orca.{moduleName}')
+    except Exception:
         try:
             factory = importlib.import_module(moduleName)
-        except:
+        except Exception:
             debug.printException(debug.LEVEL_SEVERE)
 
     # Now, get the speech server we care about.
@@ -87,23 +72,22 @@ def _initSpeechServer(moduleName, speechServerInfo):
     if not _speechserver:
         _speechserver = factory.SpeechServer.getSpeechServer()
         if speechServerInfo:
-            msg = 'SPEECH: Invalid speechServerInfo: %s' % speechServerInfo
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["SPEECH: Invalid speechServerInfo:", speechServerInfo]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if not _speechserver:
-        raise Exception("ERROR: No speech server for factory: %s" % moduleName)
+        raise Exception(f"ERROR: No speech server for factory: {moduleName}")
 
 def init():
-    debug.println(debug.LEVEL_INFO, 'SPEECH: Initializing', True)
+    debug.printMessage(debug.LEVEL_INFO, 'SPEECH: Initializing', True)
     if _speechserver:
-        debug.println(debug.LEVEL_INFO, 'SPEECH: Already initialized', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SPEECH: Already initialized', True)
         return
 
     try:
         moduleName = settings.speechServerFactory
-        _initSpeechServer(moduleName,
-                          settings.speechServerInfo)
-    except:
+        _initSpeechServer(moduleName, settings.speechServerInfo)
+    except Exception:
         moduleNames = settings.speechFactoryModules
         for moduleName in moduleNames:
             if moduleName != settings.speechServerFactory:
@@ -111,21 +95,40 @@ def init():
                     _initSpeechServer(moduleName, None)
                     if _speechserver:
                         break
-                except:
+                except Exception:
                     debug.printException(debug.LEVEL_SEVERE)
 
     if _speechserver:
-        msg = 'SPEECH: Using speech server factory: %s' % moduleName
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["SPEECH: Using speech server factory:", moduleName]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
     else:
         msg = 'SPEECH: Not available'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-    debug.println(debug.LEVEL_INFO, 'SPEECH: Initialized', True)
+    debug.printMessage(debug.LEVEL_INFO, 'SPEECH: Initialized', True)
+
+def checkSpeechSetting():
+    msg = "SPEECH: Checking speech setting."
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+    if not settings.enableSpeech:
+        shutdown()
+    else:
+        init()
 
 def __resolveACSS(acss=None):
-    if acss:
+    if isinstance(acss, ACSS):
+        family = acss.get(acss.FAMILY)
+        try:
+            family = VoiceFamily(family)
+        except Exception:
+            family = VoiceFamily({})
+        acss[acss.FAMILY] = family
         return acss
+    elif isinstance(acss, list) and len(acss) == 1:
+        return ACSS(acss[0])
+    elif isinstance(acss, dict):
+        return ACSS(acss)
     else:
         voices = settings.voices
         return ACSS(voices[settings.DEFAULT_VOICE])
@@ -137,32 +140,30 @@ def sayAll(utteranceIterator, progressCallback):
         _speechserver.sayAll(utteranceIterator, progressCallback)
     else:
         for [context, acss] in utteranceIterator:
-            logLine = "SPEECH OUTPUT: '" + context.utterance + "'"
-            debug.println(debug.LEVEL_INFO, logLine, True)
+            logLine = f"SPEECH OUTPUT: '{context.utterance}'"
+            debug.printMessage(debug.LEVEL_INFO, logLine, True)
             log.info(logLine)
 
 def _speak(text, acss, interrupt):
     """Speaks the individual string using the given ACSS."""
 
-    logLine = "SPEECH OUTPUT: '" + text + "'"
-    extraDebug = ""
-    if acss in list(settings.voices.values()):
-        for key in settings.voices:
-            if acss == settings.voices[key]:
-                if key != settings.DEFAULT_VOICE:
-                    extraDebug = " voice=%s" % key
-                break
+    if not _speechserver:
+        logLine = f"SPEECH OUTPUT: '{text}' {acss}"
+        debug.printMessage(debug.LEVEL_INFO, logLine, True)
+        log.info(logLine)
+        return
 
-    debug.println(debug.LEVEL_INFO, logLine + extraDebug + str(acss), True)
-    log.info(logLine + extraDebug)
+    voice = ACSS(settings.voices.get(settings.DEFAULT_VOICE))
+    try:
+        voice.update(__resolveACSS(acss))
+    except Exception as error:
+        msg = f"SPEECH: Exception updated voice with {acss}: {error}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-    if _speechserver:
-        voice = ACSS(settings.voices.get(settings.DEFAULT_VOICE))
-        try:
-            voice.update(acss)
-        except:
-            pass
-        _speechserver.speak(text, __resolveACSS(voice), interrupt)
+    resolvedVoice = __resolveACSS(voice)
+    msg = f"SPEECH OUTPUT: '{text}' {resolvedVoice}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+    _speechserver.speak(text, resolvedVoice, interrupt)
 
 def speak(content, acss=None, interrupt=True):
     """Speaks the given content.  The content can be either a simple
@@ -176,20 +177,35 @@ def speak(content, acss=None, interrupt=True):
                   speech_generator.LineBreak, ACSS)
     error = "SPEECH: bad content sent to speak(): '%s'"
     if not isinstance(content, validTypes):
-        debug.printStack(debug.LEVEL_WARNING)
-        debug.println(debug.LEVEL_WARNING, error % content, True)
+        debug.printMessage(debug.LEVEL_INFO, error % content, True)
         return
 
+    global _timestamp
+    if _timestamp:
+        msg = f"SPEECH: Last spoke {time.time() - _timestamp:.4f} seconds ago"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+    _timestamp = time.time()
+
+    if isinstance(content, str):
+        msg = f"SPEECH: Speak '{content}' acss: {acss}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+    else:
+        tokens = ["SPEECH: Speak", content, ", acss:", acss]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
     if isinstance(content, str):
         _speak(content, acss, interrupt)
     if not isinstance(content, list):
         return
 
     toSpeak = []
-    activeVoice = ACSS(acss)
+    activeVoice = acss
+    if acss is not None:
+        activeVoice = ACSS(acss)
+
     for element in content:
         if not isinstance(element, validTypes):
-            debug.println(debug.LEVEL_WARNING, error % element, True)
+            debug.printMessage(debug.LEVEL_INFO, error % element, True)
         elif isinstance(element, list):
             speak(element, acss, interrupt)
         elif isinstance(element, str):
@@ -199,12 +215,16 @@ def speak(content, acss=None, interrupt=True):
             newVoice = ACSS(acss)
             newItemsToSpeak = []
             if isinstance(element, speech_generator.Pause):
-                if not toSpeak[-1].endswith('.'):
+                if toSpeak[-1] and toSpeak[-1][-1].isalnum():
                     toSpeak[-1] += '.'
             elif isinstance(element, ACSS):
                 newVoice.update(element)
+                if activeVoice is None:
+                    activeVoice = newVoice
                 if newVoice == activeVoice:
                     continue
+                tokens = ["SPEECH: New voice", newVoice, " != active voice", activeVoice]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 newItemsToSpeak.append(toSpeak.pop())
 
             if toSpeak:
@@ -217,7 +237,7 @@ def speak(content, acss=None, interrupt=True):
         string = " ".join(toSpeak)
         _speak(string, activeVoice, interrupt)
 
-def speakKeyEvent(event):
+def speakKeyEvent(event, acss=None):
     """Speaks a key event immediately.
 
     Arguments:
@@ -227,15 +247,22 @@ def speakKeyEvent(event):
     if settings.silenceSpeech:
         return
 
+    global _timestamp
+    if _timestamp:
+        msg = f"SPEECH: Last spoke {time.time() - _timestamp:.4f} seconds ago"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+    _timestamp = time.time()
+
     keyname = event.getKeyName()
     lockingStateString = event.getLockingStateString()
-    msg = "%s %s" % (keyname, lockingStateString)
-    logLine = "SPEECH OUTPUT: '%s'" % msg
-    debug.println(debug.LEVEL_INFO, logLine, True)
+    acss = __resolveACSS(acss)
+    msg = f"{keyname} {lockingStateString}"
+    logLine = f"SPEECH OUTPUT: '{msg.strip()}' {acss}"
+    debug.printMessage(debug.LEVEL_INFO, logLine, True)
     log.info(logLine)
 
     if _speechserver:
-        _speechserver.speakKeyEvent(event)
+        _speechserver.speakKeyEvent(event, acss)
 
 def speakCharacter(character, acss=None):
     """Speaks a single character immediately.
@@ -251,20 +278,20 @@ def speakCharacter(character, acss=None):
     if settings.silenceSpeech:
         return
 
-    msg = "SPEECH OUTPUT: '" + character + "' " + str(acss)
-    debug.println(debug.LEVEL_INFO, msg, True)
-    log.info("SPEECH OUTPUT: '%s'" % character)
+    global _timestamp
+    if _timestamp:
+        msg = f"SPEECH: Last spoke {time.time() - _timestamp:.4f} seconds ago"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+    _timestamp = time.time()
+
+    acss = __resolveACSS(acss)
+    tokens = [f"SPEECH OUTPUT: '{character}'", acss]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
+    log.info(f"SPEECH OUTPUT: '{character}'")
 
     if _speechserver:
         _speechserver.speakCharacter(character, acss=acss)
 
-def isSpeaking():
-    """Returns True if the system is currently speaking."""
-    if _speechserver:
-        return _speechserver.isSpeaking()
-    else:
-        return False
-
 def getInfo():
     info = None
     if _speechserver:
@@ -276,59 +303,8 @@ def stop():
     if _speechserver:
         _speechserver.stop()
 
-def updateCapitalizationStyle(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.updateCapitalizationStyle()
-
-    return True
-
-def updatePunctuationLevel(script=None, inputEvent=None):
-    """ Punctuation level changed, inform this speechServer. """
-
-    if _speechserver:
-        _speechserver.updatePunctuationLevel()
-
-    return True
-
-def increaseSpeechRate(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.increaseSpeechRate()
-
-    return True
-
-def decreaseSpeechRate(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.decreaseSpeechRate()
-    else:
-        logLine = "SPEECH OUTPUT: 'slower'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
-
-    return True
-
-def increaseSpeechPitch(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.increaseSpeechPitch()
-
-    return True
-
-def decreaseSpeechPitch(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.decreaseSpeechPitch()
-
-    return True
-
-def increaseSpeechVolume(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.increaseSpeechVolume()
-    return True
-
-def decreaseSpeechVolume(script=None, inputEvent=None):
-    if _speechserver:
-        _speechserver.decreaseSpeechVolume()
-    return True
-
 def shutdown():
+    debug.printMessage(debug.LEVEL_INFO, 'SPEECH: Shutting down', True)
     global _speechserver
     if _speechserver:
         _speechserver.shutdownActiveServers()
@@ -338,31 +314,5 @@ def reset(text=None, acss=None):
     if _speechserver:
         _speechserver.reset(text, acss)
 
-def testNoSettingsInit():
-    init()
-    speak("testing")
-    speak("this is higher", ACSS({'average-pitch' : 7}))
-    speak("this is slower", ACSS({'rate' : 3}))
-    speak("this is faster", ACSS({'rate' : 80}))
-    speak("this is quiet",  ACSS({'gain' : 2}))
-    speak("this is loud",   ACSS({'gain' : 10}))
-    speak("this is normal")
-
-def test():
-    from . import speechserver
-    factories = getSpeechServerFactories()
-    for factory in factories:
-        print(factory.__name__)
-        servers = factory.SpeechServer.getSpeechServers()
-        for server in servers:
-            try:
-                print("    ", server.getInfo())
-                for family in server.getVoiceFamilies():
-                    name = family[speechserver.VoiceFamily.NAME]
-                    print("      ", name)
-                    acss = ACSS({ACSS.FAMILY : family})
-                    server.speak(name, acss)
-                    server.speak("testing")
-                server.shutdown()
-            except:
-                debug.printException(debug.LEVEL_OFF)
+def getSpeechServer():
+    return _speechserver
